#include "myh.h"
#include "SimpleAudio.h"

AudioDeviceManager *GlobalAudioDeviceManager::mAdm = 0;

AudioIODeviceCallback *GlobalAudioDeviceManager::mCurCallback = 0;

void GlobalAudioDeviceManager::setAudioCallback (AudioIODeviceCallback *newCallback)
{
	if (mCurCallback != newCallback)
		instance()->setAudioCallback(newCallback);
	mCurCallback = newCallback;
}

void GlobalAudioDeviceManager::removeAudioCallback (AudioIODeviceCallback *newCallback)
{
	if (mAdm && mCurCallback == newCallback)
	{
		mAdm->setAudioCallback(0);
		mCurCallback = 0;
	}
}

void GlobalAudioDeviceManager::init()

{
	jassert(mAdm == 0);
	delete mAdm;
	mAdm = new AudioDeviceManager();
	jassert(mAdm);

	XmlElement *prefs = 0; // :TODO: read XML preferences

	const String error (mAdm->initialise (1,        // number of input channels
                                          2,        // number of output channels
                                          prefs,    // XML settings..
                                          true  )); // select default device on failure


	if (error.isNotEmpty())
	{
		AlertWindow::showMessageBox (AlertWindow::WarningIcon,
									 T("Audio Demo"),
									 T("Couldn't open an output device!\n\n") + error);
	}
}


void GlobalAudioDeviceManager::showDialog(Component *centreAround)
{
	AudioDeviceManager &adm = *instance();

	AudioDeviceSelectorComponent audioSettingsComp (adm,
													0, 0,
													0, 6,
													false, false);

	// ...and show it in a DialogWindow...
	audioSettingsComp.setSize (480, 350);

	DialogWindow::showModalDialog (T("Audio Settings"),
								   &audioSettingsComp,
								   centreAround,
								   Colours::azure,
								   true,
								   true,
								   true);


	XmlElement *xml = adm.createStateXml();
    // :TODO: write XML preferences
}

void GlobalAudioDeviceManager::release()
{
	if (mAdm)
	{
		mAdm->setAudioCallback(0);
		deleteAndZero(mAdm);
	}
}
