#ifndef SIMPLE_AUDIO
#define SIMPLE_AUDIO


class GlobalAudioDeviceManager
{
public:

	static void setAudioCallback (AudioIODeviceCallback *newCallback);

	static void removeAudioCallback (AudioIODeviceCallback *newCallback);
	
	static void showDialog(Component *centreAround);

	static void release();

private:
	static inline AudioDeviceManager* GlobalAudioDeviceManager::instance()
	{
		if (mAdm == 0)
			init();
		return mAdm;
	}

	static void init();

	static AudioDeviceManager *mAdm;

	static AudioIODeviceCallback *mCurCallback;
};

/////////////////////////////////////////////////////////////////////////


class WavPlayer
{
	AudioFormatReaderSource *currentAudioFileSource;
	double sampleRate;

public: 
	WavPlayer() : currentAudioFileSource(0), sampleRate(44100.0) {}
	~WavPlayer() { release(); }

	inline AudioFormatReaderSource  *getAudioFormatReaderSource() { return currentAudioFileSource; }

	inline double getSampleRate() { return sampleRate; }

	// call first: AudioTransportSource::setSource(0)
	void release()
	{
		deleteAndZero(currentAudioFileSource);
	}

	// call first: AudioTransportSource::setSource(0)
	bool openFile(const File &file) 
	{
		deleteAndZero(currentAudioFileSource);
		sampleRate = 44100.0;

		WavAudioFormat waf;
		AudioFormatReader* reader = waf.createReaderFor (file.createInputStream(), true);

		if (reader == 0)
			return false;

		sampleRate = reader->sampleRate;

		currentAudioFileSource = new AudioFormatReaderSource (reader, true);

		return currentAudioFileSource != 0;
	}
};

//////////////////////////////////////////////////////////////////////////

class WavPlayerComplete
{
	AudioSourcePlayer audioSourcePlayer;
	AudioTransportSource audioTransportSource;
	WavPlayer wavPlayer;

public:

	~WavPlayerComplete() { release(); }

	void release()
	{
		audioTransportSource.stop();
		audioTransportSource.setSource(0);
		audioSourcePlayer.setSource(0);
		GlobalAudioDeviceManager::removeAudioCallback(&audioSourcePlayer);
	}

	bool openFile(const File &file) 
	{
		audioTransportSource.setSource(0);
		audioSourcePlayer.setSource(0);

		GlobalAudioDeviceManager::setAudioCallback(&audioSourcePlayer);
		audioSourcePlayer.setSource(&audioTransportSource);
		
		if (!wavPlayer.openFile(file))
			return false;

		audioTransportSource.setSource(wavPlayer.getAudioFormatReaderSource(),
                                       32768,
                                       wavPlayer.getSampleRate()); 

		return true;
	}

	void play(bool playFromStart) 
	{ 
		if (playFromStart)
			audioTransportSource.setPosition(0.0);

		if (!audioTransportSource.isPlaying())
			audioTransportSource.start();
	}

	void stop()
	{
		audioTransportSource.stop();
	}
};

#endif